@@ -14,14 +14,14 @@
 
 import sqlite3
 from os import remove
-from os.path import (splitext, split)
+from os.path import (splitext, split, exists)
 import logging
 
 import numpy as np
 from Bio import SeqIO
 
-from common import build_path
-from download import fetch_file, fetch_csv_dataframe, normalize_filename
+from common import build_path, normalize_filename
+from download import fetch_file, fetch_csv_dataframe
 
 
 def db_table_exists(db, table_name):
@@ -88,7 +88,7 @@ def _dtype_to_db_type(dtype):
 
     assert False, "Failed to find sqlite3 column type for %s" % dtype
 
-def create_db(db, table_name, col_types, rows, primary_key = None):
+def create_table(db, table_name, col_types, primary_key = None, nullable=[]):
     """
     Creates a sqlite3 database from the given Python values.
 
@@ -102,6 +102,12 @@ def create_db(db, table_name, col_types, rows, primary_key = None):
 
     rows : list of tuples
         Must have as many elements in each tuple as there were col_types
+
+    primary_key : str, optional
+        Which column is the primary key
+
+    nullable : set/list, optional
+        Any column in this collection can have null values
     """
 
     col_decls = []
@@ -109,21 +115,51 @@ def create_db(db, table_name, col_types, rows, primary_key = None):
         decl = "%s %s" % (col_name,t)
         if col_name == primary_key:
             decl += " UNIQUE PRIMARY KEY"
-        decl += " NOT NULL"
+        if col_name not in nullable:
+            decl += " NOT NULL"
         col_decls.append(decl)
     col_decl_str = ", ".join(col_decls)
     create = \
         "create table %s (%s)" % (table_name, col_decl_str)
     logging.info("Running sqlite query: \"%s\"", create)
     db.execute(create)
-
-    blank_slots = ", ".join("?" for _ in col_types)
-    logging.info("Inserting %d rows into table %s", len(rows), table_name)
-    db.executemany("insert into %s values (%s)" % (table_name, blank_slots), rows)
     db.commit()
-    return db
 
-def create_cached_db(db_filename, table_name, fn, subdir = None):
+def fill_table(db, table_name, rows):
+    assert len(rows) > 0
+    row = rows[0]
+
+    n_cols = len(row)
+    blank_slots = ", ".join("?" for _ in xrange(n_cols))
+    logging.info("Inserting %d rows into table %s", len(rows), table_name)
+    db.executemany(
+        "insert into %s values (%s)" % (table_name, blank_slots),
+        rows)
+    db.commit()
+
+def create_db_indices(db, table_name, indices):
+    for i, index_col_set in enumerate(indices):
+        logging.info("Creating index on %s (%s)" % (
+            table_name,
+            ", ".join(index_col_set)
+        ))
+        index_name = "index%d_%s" % (i, "_".join(index_col_set))
+        db.execute(
+            "CREATE INDEX IF NOT EXISTS %s ON %s (%s)" % (
+                index_name,
+                table_name,
+                ", ".join(index_col_set)
+            )
+        )
+    db.commit()
+
+def create_cached_db(
+        db_filename,
+        table_name,
+        fn,
+        subdir = None,
+        nullable = [],
+        indices = []):
     """
     Either create or retrieve sqlite database.
 
@@ -137,6 +173,12 @@ def create_cached_db(db_filename, table_name, fn, subdir = None):
     fn : function
         Returns (rows, col_types, key_column_name)
 
+    subdir : str
+
+    nullable : list/seq
+
+    indices : list of string tuples
+
     """
     db_path = build_path(db_filename, subdir)
 
@@ -150,19 +192,25 @@ def create_cached_db(db_filename, table_name, fn, subdir = None):
         if db_table_exists(db, table_name):
             logging.info("Found existing table in database %s", db_path)
         else:
-            logging.info("Creating database table %s at %s", table_name, db_path)
+            logging.info(
+                "Creating database table %s at %s", table_name, db_path)
             col_types, rows, key_column_name = fn()
-            db = create_db(db, table_name, col_types, rows, primary_key = key_column_name)
+            create_table(
+                db, table_name, col_types,
+                primary_key = key_column_name,
+                nullable = nullable)
+            fill_table(db, table_name, rows)
+            create_db_indices(db, table_name, indices)
     except:
-        logging.warning("Failed to create table %s in database %s", table_name, db_path)
+        logging.warning(
+            "Failed to create table %s in database %s",
+            table_name, db_path)
         db.close()
         remove(db_path)
         raise
     return db
 
 
-
-
 def fetch_fasta_db(
         table_name,
         download_url,
@@ -198,33 +246,71 @@ def fetch_fasta_db(
         ]
         return col_types, rows, key_column
 
-    return create_cached_db(db_filename, table_name, fn = load_data, subdir = subdir)
+    return create_cached_db(
+        db_filename,
+        table_name,
+        fn = load_data,
+        subdir = subdir)
 
-
-def db_from_dataframe(base_filename, table_name, df, key_column_name = None, subdir = None):
+def construct_db_filename(base_filename, df):
     """
-    Given a dataframe `df`, turn it into a sqlite3 database.
-    Use `base_filename` as the root of the local db filename and store
-    values in a table called `table_name`.
+    Generate  filename for a DataFeame
     """
-
-    # tag cached database by dataframe's number of rows and columns
     db_filename = base_filename + ("_nrows%d" % len(df))
+    for col_name in df.columns:
+        col_db_type = _dtype_to_db_type(col.dtype)
+        col_name = col_name.replace(" ", "_")
+        db_filename += ".%s_%s" % (col_name, col_db_type)
+    return db_filename + ".db"
+
+def db_from_dataframe(
+        db_filename,
+        table_name,
+        df,
+        key_column_name = None,
+        subdir = None,
+        overwrite = False,
+        indices = []):
+    """
+    Given a dataframe `df`, turn it into a sqlite3 database.
+    Store values in a table called `table_name`.
+    """
+    db_path = build_path(db_filename, subdir)
+
+    if overwrite  and exists(db_path):
+        remove(db_path)
+
+    col_types = []
+    nullable = set([])
+
+    # tag cached database by dataframe's number of rows and columns
     for col_name in df.columns:
         col = df[col_name]
+        if col.isnull().any():
+            nullable.add(col_name)
         col_db_type = _dtype_to_db_type(col.dtype)
         col_name = col_name.replace(" ", "_")
         col_types.append( (col_name, col_db_type) )
-        db_filename += ".%s_%s" % (col_name, col_db_type)
-    db_filename += ".db"
-    db_path = build_path(db_filename, subdir)
 
     def create_rows():
         rows = list(tuple(row) for row in df.values)
         return col_types, rows, key_column_name
-    return create_cached_db(db_path, table_name, create_rows, subdir = subdir)
 
-def fetch_csv_db(table_name, download_url, csv_filename = None, subdir = None, **pandas_kwargs):
+    return create_cached_db(
+        db_path,
+        table_name,
+        create_rows,
+        subdir = subdir,
+        nullable = nullable,
+        indices = indices)
+
+def fetch_csv_db(
+        table_name,
+        download_url,
+        csv_filename = None,
+        db_filename = None,
+        subdir = None,
+        **pandas_kwargs):
     """
     Download a remote CSV file and create a local sqlite3 database from its contents
     """
@@ -234,4 +320,6 @@ def fetch_csv_db(table_name, download_url, csv_filename = None, subdir = None, *
         subdir = subdir,
         **pandas_kwargs)
     base_filename = splitext(csv_filename)[0]
-    return db_from_dataframe(base_filename, table_name, df, subdir = subdir)
\ No newline at end of file
+    if db_filename is None:
+        db_filename = construct_db_filename(base_filename, df)
+    return db_from_dataframe(db_filename, table_name, df, subdir = subdir)
\ No newline at end of file
